@@ -33,7 +33,7 @@ from duplicity import globals  # @Reimport
 from duplicity import diffdir
 from duplicity import util  # @Reimport
 from duplicity.globmatch import GlobbingError, FilePrefixError, \
-    path_matches_glob
+    path_matches_glob_fn
 
 """Iterate exactly the requested files in a directory
 
@@ -447,10 +447,24 @@ probably isn't what you meant.""") %
 
         def exclude_sel_func(path):
             # do not follow symbolic links when checking for file existence!
-            if path.isdir() and path.append(filename).exists():
-                return 0
-            else:
-                return None
+            if path.isdir():
+                # First check path is read accessible
+                if not (os.access(path.name, os.R_OK)):
+                    # Path is not read accessible
+                    # ToDo: Ideally this error would only show if the folder
+                    # was ultimately included by the full set of selection
+                    # functions. Currently this will give an error for any
+                    # locked directory within the folder being backed up.
+                    log.Warn(_(
+                        "Error accessing possibly locked file %s") % util.ufn(
+                        path.name),
+                        log.WarningCode.cannot_read, util.escape(path.name))
+                    if diffdir.stats:
+                        diffdir.stats.Errors += 1
+                elif path.append(filename).exists():
+                    return 0
+                else:
+                    return None
 
         if include == 0:
             sel_func = exclude_sel_func
@@ -473,6 +487,9 @@ probably isn't what you meant.""") %
 
         """
         # Internal. Used by glob_get_sf and unit tests.
+        # ToDo: Make all globbing/non-globbing use same code path
+        # This distinction has bitten us too many times with bugs in one or
+        # the other.
         match_only_dirs = False
 
         if filename != "/" and filename[-1] == "/":
@@ -492,8 +509,10 @@ probably isn't what you meant.""") %
         # Internal. Used by glob_get_filename_sf.
 
         def include_sel_func(path):
-            if match_only_dirs and not path.isdir():
-                # If the glob ended with a /, only match directories
+            if len(tuple) == len(path.index) and match_only_dirs and not path.isdir():
+                # If we are assessing the actual directory (rather than the
+                # contents of the directory) and the glob ended with a /,
+                # only match directories
                 return None
             elif (path.index == tuple[:len(path.index)] or
                     path.index[:len(tuple)] == tuple):
@@ -544,13 +563,10 @@ probably isn't what you meant.""") %
             ignore_case = True
 
         # Check to make sure prefix is ok
-        if not path_matches_glob(self.rootpath, glob_str, include=1):
+        if not path_matches_glob_fn(glob_str, include=1)(self.rootpath):
             raise FilePrefixError(glob_str)
 
-        def sel_func(path):
-            return path_matches_glob(path, glob_str, include, ignore_case)
-
-        return sel_func
+        return path_matches_glob_fn(glob_str, include, ignore_case)
 
     def exclude_older_get_sf(self, date):
         """Return selection function based on files older than modification date """
